/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.annotation.valueprovider;

import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;

/**
 * Container class to uniquely specify an {@link IAnnotatedSpecification} including its instnaceKey
 * of multi-valued {@link IAnnotatedSpecification}s.
 * 
 * @author diewald
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating YELLOW Hash: 41A961F3E903456E5DD7A9867994DA8C
 */
public class AnnotationInstSpec<T extends IAnnotatedSpecification> {
	/**
	 * {@link IAnnotatedSpecification} from {@code entry} to be displayed in the column
	 * represented by this {@link AnnotationInstSpec}.
	 */
	private Class<T> annotatedSpecificationClass;

	/**
	 * Instance key identifying the particular instance of an {@link IAnnotatedSpecification} from
	 * {@code entry} to be displayed in the column represented by this {@link AnnotationInstSpec}.
	 */
	private String instanceKey;

	/** Constructor. */
	public AnnotationInstSpec(Class<T> annotatedSpecificationClass, String instanceKey) {
		this.annotatedSpecificationClass = annotatedSpecificationClass;
		this.instanceKey = instanceKey;
	}

	/**
	 * Returns the {@link IAnnotatedSpecification} from {@code entry} to be displayed in the
	 * column represented by this {@link AnnotationInstSpec}.
	 */
	public Class<T> getAnnotatedSpecificationType() {
		return annotatedSpecificationClass;
	}

	/**
	 * Returns the instance key identifying the particular instance of an
	 * {@link IAnnotatedSpecification} from {@code entry} to be displayed in the column
	 * represented by this {@link AnnotationInstSpec}.
	 */
	public String getInstanceKey() {
		return instanceKey;
	}

	/** {@inheritDoc} */
	@Override
	public int hashCode() {
		return annotatedSpecificationClass.hashCode() +
				(instanceKey != null ? instanceKey.hashCode() : 0);
	}

	/** {@inheritDoc} */
	@Override
	public boolean equals(Object other) {
		if(other instanceof AnnotationInstSpec) {
			return this.hashCode() == other.hashCode();
		}
		return super.equals(other);
	}
}
