/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.annotation.valueprovider;

import static org.fortiss.tooling.kernel.utils.EcoreUtils.getChildrenWithType;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.function.BiConsumer;

import org.conqat.lib.commons.collections.Pair;
import org.fortiss.tooling.base.ToolingBaseActivator;
import org.fortiss.tooling.base.annotation.AnnotationEntry;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.service.IPersistencyService;
import org.fortiss.tooling.kernel.utils.LoggingUtils;

/**
 * Interface defining default implementations for the context menu of
 * {@link IAnnotatedSpecification}s.
 * 
 * @author diewald
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating YELLOW Hash: 5A3C99F6C95BD4E0AFE4B8B2D35C0752
 */
public interface DefaultAnnotationContextActions<T extends IAnnotatedSpecification> extends
		IAnnotationValueProvider<T> {

	/** {@inheritDoc} */
	@Override
	public default List<Pair<String, BiConsumer<AnnotationInstSpec<T>, AnnotationEntry>>>
			getContextMenuEntries() {
		List<Pair<String, BiConsumer<AnnotationInstSpec<T>, AnnotationEntry>>> retList =
				new ArrayList<>();

		// Propagates values from the selected entry to all entries in the same ITopLevelElement
		// with the same type.
		BiConsumer<AnnotationInstSpec<T>, AnnotationEntry> propagateToAll =
				(AnnotationInstSpec<T> spec, AnnotationEntry entry) -> {

					Class<T> specType = spec.getAnnotatedSpecificationType();
					Object value = entry.getSpecificationValue(specType);

					ITopLevelElement topElement =
							IPersistencyService.getInstance().getTopLevelElementFor(
									entry.getModelElement());

					Collection<T> sameAnnotations =
							getChildrenWithType(topElement.getRootModelElement(), specType);
					for(T sameAnn : sameAnnotations) {
						// Apply the value for each annotation of the same type in the smae
						// IToplevelElement.
						Runnable run =
								() -> {
									try {
										setAnnotationValue(value, sameAnn);
									} catch(Exception ex) {
										LoggingUtils
												.error(ToolingBaseActivator.getDefault(),
														"Could not set the value of the annotation " +
																sameAnn.getName() +
																" when trying to propagate the value form the selected annotation. Reason:\n" +
																ex.getLocalizedMessage());
									}
								};
						topElement.runAsCommand(run);
					}
				};
		// Add the propagate action to the list of context menu entries.
		retList.add(new Pair<String, BiConsumer<AnnotationInstSpec<T>, AnnotationEntry>>(
				"Propagate value", propagateToAll));

		return retList;
	}
}
